@@ -87,16 +87,28 @@ class SyncManager:
             del(self.syncWorkers[cond])
 
     def _start_worker_thread(self, cond, worker):
+        log.info("Setting global cancel flag")
+        conduit.GLOBALS.cancelled = False
+
         log.debug("Starting worker: %s" % worker)
         cond.connect("sync-completed", self._on_sync_completed, worker)
         self.syncWorkers[cond] = worker
         self.syncWorkers[cond].start()
+        
+    def is_busy(self):
+        """
+        Returns true if any conduit is currently undergoing a sync
+        """
+        for cond in self.syncWorkers:
+            if self.syncWorkers[cond].isAlive():
+                return True
+        return False
 
     def sync_in_progress(self, cond):
         """
         Returns true if cond is currently undergoing sync, refresh etc
         """
-        return cond in self.syncWorkers
+        return cond in self.syncWorkers and self.syncWorkers[cond].isAlive()
 
     def cancel_all(self):
         """
@@ -119,12 +131,32 @@ class SyncManager:
         for c in self.syncWorkers:
             self.syncWorkers[c].join(timeout)
 
-    def refresh_dataprovider(self, cond, dataprovider):
+    def run_blocking_dataprovider_function_calls(self, dataprovider, callback, *functions):
+        #need to get the conduit assocated with this dataprovider because the sync-completed
+        #signal is emmited from the conduit object
+        conds = []
+        conds.extend(conduit.GLOBALS.app.guiSyncSet.get_all_conduits())
+        conds.extend(conduit.GLOBALS.app.dbusSyncSet.get_all_conduits())
+        for c in conds:
+            for dpw in c.get_all_dataproviders():
+                if dataprovider == dpw.module:
+                    #found it!
+                    if c not in self.syncWorkers:
+                        #connect the supplied callback
+                        c.connect("sync-completed",callback)
+                        #start the thread
+                        bfcw = BlockingFunctionCallWorker(c, *functions)
+                        self._start_worker_thread(c, bfcw)
+                    return
+
+        log.info("Could not create BlockingFunctionCallWorker")            
+
+    def refresh_dataprovider(self, cond, dataproviderWrapper):
         if cond in self.syncWorkers:
-            log.info("Refresh dataprovider already in progress")
+            log.info("Refresh dataproviderWrapper already in progress")
             self.join_one(cond)            
 
-        threadedWorker = RefreshDataProviderWorker(cond, dataprovider)
+        threadedWorker = RefreshDataProviderWorker(cond, dataproviderWrapper)
         self._start_worker_thread(cond, threadedWorker)
 
     def refresh_conduit(self, cond):
@@ -240,6 +272,9 @@ class SyncWorker(_ThreadedWorker):
     Operates on a per Conduit basis, so a single SyncWorker may synchronize
     one source with many sinks within a single conduit
     """
+
+    PROGRESS_UPDATE_THRESHOLD = 5.0/100
+
     def __init__(self, typeConverter, cond, do_sync):
         _ThreadedWorker.__init__(self)
         self.typeConverter = typeConverter
@@ -248,11 +283,28 @@ class SyncWorker(_ThreadedWorker):
         self.sinks = cond.datasinks
         self.do_sync = do_sync
 
+        self._progress = 0
+        self._progressUIDs = []
+        
+
         if self.cond.is_two_way():
             self.setName("%s <--> %s" % (self.source, self.sinks[0]))
         else:
             self.setName("%s |--> %s" % (self.source, self.sinks))
 
+    def _emit_progress(self, progress, dataUID):
+        """
+        Emits progress signals, if the elapsed progress since the last 
+        call to this function is greater that 5%. This is necessary because
+        otherwise we starve the main loop with too frequent progress
+        events
+        """
+        self._progressUIDs.append(dataUID)
+        if (progress - self._progress) > self.PROGRESS_UPDATE_THRESHOLD or progress == 1.0:
+            self._progress = progress
+            self.cond.emit("sync-progress", self._progress, self._progressUIDs)
+            self._progressUIDs = []
+
     def _get_data(self, source, sink, uid):
         """
         Gets the data from source. Handles exceptions, etc.
@@ -265,30 +317,22 @@ class SyncWorker(_ThreadedWorker):
         except Exceptions.SyncronizeError, err:
             log.warn("%s\n%s" % (err, traceback.format_exc()))                     
             self.sinkErrors[sink] = DataProvider.STATUS_DONE_SYNC_ERROR
-            log.debug("ERROR GETTING DATA")
-        except Exceptions.SyncronizeFatalError, err:
-            log.warn("%s\n%s" % (err, traceback.format_exc()))
-            sink.module.set_status(DataProvider.STATUS_DONE_SYNC_ERROR)                                  
-            source.module.set_status(DataProvider.STATUS_DONE_SYNC_ERROR)                             
-            #Cannot continue
-            raise Exceptions.StopSync(self.state)                  
-        except Exception:       
-            #Cannot continue
-            log.critical("UNKNOWN SYNCHRONIZATION ERROR\n%s" % traceback.format_exc())
-            sink.module.set_status(DataProvider.STATUS_DONE_SYNC_ERROR)
-            source.module.set_status(DataProvider.STATUS_DONE_SYNC_ERROR)
-            raise Exceptions.StopSync(self.state)
-
         return data
 
     def _put_data(self, source, sink, sourceData, sourceDataRid):
         """
         Handles exceptions when putting data from source to sink. Default is
         not to overwrite
+
+        @returns: True if the data was successfully put
         """
         if sourceData != None:
             try:
                 put_data(source, sink, sourceData, sourceDataRid, False)
+                return True
+            except Exceptions.SyncronizeError, err:
+                log.warn("%s\n%s" % (err, traceback.format_exc()))                     
+                self.sinkErrors[sink] = DataProvider.STATUS_DONE_SYNC_ERROR
             except Exceptions.SynchronizeConflictError, err:
                 comp = err.comparison
                 if comp == COMPARISON_OLDER:
@@ -300,6 +344,8 @@ class SyncWorker(_ThreadedWorker):
                     self._apply_conflict_policy(source, sink, err.comparison, sourceData, sourceDataRid, err.toData, err.toData.get_rid())
         else:
             log.info("Could not put data: Was None")
+        
+        return False
 
     def _convert_data(self, source, sink, data):
         """
@@ -316,9 +362,7 @@ class SyncWorker(_ThreadedWorker):
             self.sinkErrors[sink] = DataProvider.STATUS_DONE_SYNC_ERROR
         except Exception:       
             log.critical("UNKNOWN CONVERSION ERROR\n%s" % traceback.format_exc())
-            sink.module.set_status(DataProvider.STATUS_DONE_SYNC_ERROR)
-            source.module.set_status(DataProvider.STATUS_DONE_SYNC_ERROR)
-            raise Exceptions.StopSync(self.state)
+            self.sinkErrors[sink] = DataProvider.STATUS_DONE_SYNC_ERROR
         return newdata
 
     def _apply_deleted_policy(self, sourceWrapper, sourceDataLUID, sinkWrapper, sinkDataLUID):
@@ -371,43 +415,38 @@ class SyncWorker(_ThreadedWorker):
         Applies user policy when a put() has failed. This may mean emitting
         the conflict up to the GUI or skipping altogether
         """
-        if comparison == COMPARISON_EQUAL or comparison == COMPARISON_UNKNOWN or comparison == COMPARISON_OLDER:
-            if self.cond.get_policy("conflict") == "skip":
-                log.debug("Conflict Policy: Skipping")
-            elif self.cond.get_policy("conflict") == "ask":
-                log.debug("Conflict Policy: Ask")
-                self.sinkErrors[sinkWrapper] = DataProvider.STATUS_DONE_SYNC_CONFLICT
+        if self.cond.get_policy("conflict") == "skip":
+            log.debug("Conflict Policy: Skipping")
+        elif self.cond.get_policy("conflict") == "ask":
+            log.debug("Conflict Policy: Ask")
+            self.sinkErrors[sinkWrapper] = DataProvider.STATUS_DONE_SYNC_CONFLICT
 
-                if sourceWrapper.module_type in ["twoway", "sink"]:
-                    #in twoway case the user can copy back
-                    avail = (CONFLICT_SKIP,CONFLICT_COPY_SOURCE_TO_SINK,CONFLICT_COPY_SINK_TO_SOURCE)
-                else:
-                    avail = (CONFLICT_SKIP,CONFLICT_COPY_SOURCE_TO_SINK)
+            if sourceWrapper.module_type in ["twoway", "sink"]:
+                #in twoway case the user can copy back
+                avail = (CONFLICT_SKIP,CONFLICT_COPY_SOURCE_TO_SINK,CONFLICT_COPY_SINK_TO_SOURCE)
+            else:
+                avail = (CONFLICT_SKIP,CONFLICT_COPY_SOURCE_TO_SINK)
 
-                c = Conflict(
-                        sourceWrapper, 
-                        fromData,
-                        fromDataRid, 
-                        sinkWrapper, 
-                        toData,
-                        toDataRid,
-                        avail,
-                        False
-                        )
-                self.cond.emit("sync-conflict", c)
+            c = Conflict(
+                    sourceWrapper, 
+                    fromData,
+                    fromDataRid, 
+                    sinkWrapper, 
+                    toData,
+                    toDataRid,
+                    avail,
+                    False
+                    )
+            self.cond.emit("sync-conflict", c)
 
-            elif self.cond.get_policy("conflict") == "replace":
-                log.debug("Conflict Policy: Replace")
-                self.sinkErrors[sinkWrapper] = DataProvider.STATUS_DONE_SYNC_CONFLICT
+        elif self.cond.get_policy("conflict") == "replace":
+            log.debug("Conflict Policy: Replace")
+            self.sinkErrors[sinkWrapper] = DataProvider.STATUS_DONE_SYNC_CONFLICT
 
-                try:
-                    put_data(sourceWrapper, sinkWrapper, fromData, fromDataRid, True)
-                except:
-                    log.warn("Forced Put Failed\n%s" % traceback.format_exc())        
-        #This should not happen...
-        else:
-            log.warn("UNKNOWN COMPARISON\n%s" % traceback.format_exc())
-            self.sinkErrors[sinkWrapper] = DataProvider.STATUS_DONE_SYNC_ERROR
+            try:
+                put_data(sourceWrapper, sinkWrapper, fromData, fromDataRid, True)
+            except:
+                log.warn("Forced Put Failed\n%s" % traceback.format_exc())        
 
     def check_thread_not_cancelled(self, dataprovidersToCancel):
         """
@@ -443,11 +482,6 @@ class SyncWorker(_ThreadedWorker):
             idx += 1.0
             self.check_thread_not_cancelled([source, sink])
 
-            #work out the percent complete
-            done = idx/(numItems*len(self.sinks)) + \
-                    float(self.sinks.index(sink))/len(self.sinks)
-            self.cond.emit("sync-progress", done)
-
             #transfer the data
             data = self._get_data(source, sink, i)
             if data != None:
@@ -455,6 +489,11 @@ class SyncWorker(_ThreadedWorker):
                 dataRid = data.get_rid()
                 data = self._convert_data(source, sink, data)
                 self._put_data(source, sink, data, dataRid)
+
+            #work out the percent complete
+            done = idx/(numItems*len(self.sinks)) + \
+                    float(self.sinks.index(sink))/len(self.sinks)
+            self._emit_progress(done, i)
        
     def two_way_sync(self, source, sink):
         """
@@ -521,7 +560,7 @@ class SyncWorker(_ThreadedWorker):
 
             #progress
             cnt = cnt+1
-            self.cond.emit("sync-progress", float(cnt)/total)
+            self._emit_progress(float(cnt)/total, dataUID)
 
         for sourcedp, dataUID, sinkdp in toput:
             data = self._get_data(sourcedp, sinkdp, dataUID)
@@ -532,7 +571,7 @@ class SyncWorker(_ThreadedWorker):
                 self._put_data(sourcedp, sinkdp, data, dataRid)
 
             cnt = cnt+1
-            self.cond.emit("sync-progress", float(cnt)/total)
+            self._emit_progress(float(cnt)/total, dataUID)
 
         #FIXME: rename dp1 -> sourcedp1 and dp2 -> sinkdp2 because when both
         #data is modified we might as well choost source -> sink as the comparison direction
@@ -548,9 +587,6 @@ class SyncWorker(_ThreadedWorker):
             
             log.debug("2WAY CMP: %s v %s" % (data1, data2))
 
-            cnt = cnt+1
-            self.cond.emit("sync-progress", float(cnt)/total)
-
             #compare the data
             if data1 != None and data2 != None:
                 comparison = data1.compare(data2)
@@ -559,6 +595,10 @@ class SyncWorker(_ThreadedWorker):
                 else:
                     self._apply_conflict_policy(dp1, dp2, COMPARISON_UNKNOWN, data1, data1Rid, data2, data2Rid)
 
+            cnt = cnt+1
+            self._emit_progress(float(cnt)/total, data1UID)
+
+
     def run(self):
         """
         The main syncronisation state machine.
@@ -632,7 +672,7 @@ class SyncWorker(_ThreadedWorker):
 
                 #refresh state
                 elif self.state is self.REFRESH_STATE:
-                    log.debug("Source Status = %s" % self.source.module.get_status_text())
+                    log.debug("Source Status = %s" % self.source.module.get_status())
                     #Refresh the source
                     try:
                         self.source.module.refresh()
@@ -684,15 +724,28 @@ class SyncWorker(_ThreadedWorker):
                         self.check_thread_not_cancelled([self.source, sink])
                         #only sync with those sinks that refresh'd OK
                         if sink not in sinkDidntRefreshOK:
-                            #now perform a one or two way sync depending on the user prefs
-                            #and the capabilities of the dataprovider
-                            if  self.cond.is_two_way():
-                                #two way
-                                self.two_way_sync(self.source, sink)
-                            else:
-                                #one way
-                                self.one_way_sync(self.source, sink)
-     
+                            try:
+                                #now perform a one or two way sync depending on the user prefs
+                                #and the capabilities of the dataprovider
+                                if  self.cond.is_two_way():
+                                    #two way
+                                    self.two_way_sync(self.source, sink)
+                                else:
+                                    #one way
+                                    self.one_way_sync(self.source, sink)
+                            except Exceptions.SyncronizeFatalError, err:
+                                log.warn("%s\n%s" % (err, traceback.format_exc()))
+                                sink.module.set_status(DataProvider.STATUS_DONE_SYNC_ERROR)                                  
+                                self.source.module.set_status(DataProvider.STATUS_DONE_SYNC_ERROR)                             
+                                #cannot continue with this source, sink pair
+                                continue
+                            except Exception:       
+                                log.critical("UNKNOWN SYNCHRONIZATION ERROR\n%s" % traceback.format_exc())
+                                sink.module.set_status(DataProvider.STATUS_DONE_SYNC_ERROR)
+                                self.source.module.set_status(DataProvider.STATUS_DONE_SYNC_ERROR)
+                                #cannot continue with this source, sink pair
+                                continue
+
                     #Done go clean up
                     self.state = self.DONE_STATE
 
@@ -789,7 +842,30 @@ class RefreshDataProviderWorker(_ThreadedWorker):
         conduit.GLOBALS.mappingDB.save()
         self.cond.emit("sync-completed", self.aborted, self.did_sync_error(), self.did_sync_conflict())
 
-                
+class BlockingFunctionCallWorker(_ThreadedWorker):
+    """
+    Calls the provided (blocking) function in a new thread. When
+    the function returns a sync-completed signal is sent
+    """
+    def __init__(self, cond, *functions):
+        _ThreadedWorker.__init__(self)
+        self.cond = cond
+        self.functions = functions
+        self.setName("%s functions" % len(self.functions))
+
+    def run(self):
+        try:
+            #FIXME: Set the status text on the dataprovider
+            for f in self.functions:
+                log.debug("FunctionCall %s beginning" % f.__name__)
+                f()
+            self.aborted = False
+        except Exception, e:
+            log.warn("FunctionCall error: %s" % e)
+            self.aborted = True
+
+        self.cond.emit("sync-completed", self.aborted, False, False)
+
 class DeletedData(DataType.DataType):
     """
     Simple wrapper around a deleted item. If an item has been deleted then
