@@ -40,14 +40,14 @@ USER_DIR = os.path.join(
     			"conduit"
     			)
 if not os.path.exists(USER_DIR):
-    os.mkdir(USER_DIR)
+    os.makedirs(USER_DIR)
 
 if IS_INSTALLED:
     from defs import *
     if not PYTHONDIR in sys.path:
         sys.path.insert(0, PYTHONDIR)
 else:
-    VERSION =                   "0.3.9"
+    VERSION =                   "0.3.11"
     LOCALE_DIR =                os.path.join(DIRECTORY, "po")
     SHARED_DATA_DIR =           os.path.join(DIRECTORY, "data")
     GLADE_FILE =                os.path.join(DIRECTORY, "data","conduit.glade")
