@@ -13,23 +13,21 @@ log = logging.getLogger("dataproviders.DataProvider")
 
 import conduit
 import conduit.ModuleWrapper as ModuleWrapper
-import conduit.Utils as Utils
+import conduit.utils as Utils
 import conduit.Settings as Settings
 
-#Constants used in the sync state machine
-STATUS_NONE = 0
-STATUS_CHANGE_DETECTED = 1
-STATUS_REFRESH = 2
-STATUS_DONE_REFRESH_OK = 3
-STATUS_DONE_REFRESH_ERROR = 4
-STATUS_SYNC = 5
-STATUS_DONE_SYNC_OK = 6
-STATUS_DONE_SYNC_ERROR = 7
-STATUS_DONE_SYNC_SKIPPED = 8
-STATUS_DONE_SYNC_CANCELLED = 9
-STATUS_DONE_SYNC_CONFLICT = 10
-STATUS_DONE_SYNC_NOT_CONFIGURED = 11
-
+STATUS_NONE = _("Ready")
+STATUS_CHANGE_DETECTED = _("New data to sync")
+STATUS_REFRESH = _("Refreshing...")
+STATUS_DONE_REFRESH_OK = _("Refreshed OK")
+STATUS_DONE_REFRESH_ERROR = _("Error Refreshing")
+STATUS_SYNC = _("Synchronizing...")
+STATUS_DONE_SYNC_OK = _("Synchronized OK")
+STATUS_DONE_SYNC_ERROR = _("Error Synchronizing")
+STATUS_DONE_SYNC_SKIPPED = _("Synchronization Skipped")
+STATUS_DONE_SYNC_CANCELLED = _("Synchronization Cancelled")
+STATUS_DONE_SYNC_CONFLICT = _("Synchronization Conflict")
+STATUS_DONE_SYNC_NOT_CONFIGURED = _("Not Configured Correctly")
 
 class DataProviderBase(gobject.GObject):
     """
@@ -40,8 +38,16 @@ class DataProviderBase(gobject.GObject):
                     "status-changed": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
                     "change-detected": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [])
                     }
+                    
+    _name_ = ""
+    _description_ = ""
+    _icon_ = ""
+    _module_type_ = "dataprovider"
+    _category_ = conduit.dataproviders.CATEGORY_TEST
+    _out_type_ = ""
+    _in_type_ = ""
     
-    def __init__(self):
+    def __init__(self, *args):
         """
         All sync functionality should be provided by derived classes
         """
@@ -134,9 +140,6 @@ class DataProviderBase(gobject.GObject):
         """
         Sets the dataprovider status. If the status has changed then emits
         a status-changed signal
-        
-        @param newStatus: The new status
-        @type newStatus: C{int}
         """
         if newStatus != self.get_status():
             self.status = newStatus
@@ -144,44 +147,10 @@ class DataProviderBase(gobject.GObject):
         
     def get_status(self):
         """
-        @returns: The current dataproviders status (code)
-        @rtype: C{int}
+        @returns: The current dataproviders status
         """
         return self.status
         
-    def get_status_text(self):
-        """
-        @returns: a textual representation of the current dataprover status
-        @rtype: C{str}
-        """
-        s = self.get_status()
-        if s == STATUS_NONE:
-            return _("Ready")
-        elif s == STATUS_CHANGE_DETECTED:
-            return _("New data to sync")
-        elif s == STATUS_REFRESH:
-            return _("Refreshing...")
-        elif s == STATUS_DONE_REFRESH_OK:
-            return _("Refreshed OK")
-        elif s == STATUS_DONE_REFRESH_ERROR:
-            return _("Error Refreshing")
-        elif s == STATUS_SYNC:
-            return _("Synchronizing...")
-        elif s == STATUS_DONE_SYNC_OK:
-            return _("Synchronized OK")
-        elif s == STATUS_DONE_SYNC_ERROR:
-            return _("Error Synchronizing")
-        elif s == STATUS_DONE_SYNC_SKIPPED:
-            return _("Synchronization Skipped")
-        elif s == STATUS_DONE_SYNC_CANCELLED:
-            return _("Synchronization Cancelled")
-        elif s == STATUS_DONE_SYNC_CONFLICT:
-            return _("Synchronization Conflict")
-        elif s == STATUS_DONE_SYNC_NOT_CONFIGURED:
-            return _("Not Configured Correctly")
-        else:
-            return "BAD PROGRAMMER"
-            
     def is_busy(self):
         """
         A DataProvider is busy if it is currently in the middle of the
@@ -201,12 +170,9 @@ class DataProviderBase(gobject.GObject):
     def configure(self, window):
         """
         Show a configuration box for configuring the dataprovider instance.
-        This call may block
-        
-        @param window: The parent window (to show a modal diaconduit.log)
-        @type window: {gtk.Window}
+        @param window: The parent gtk.Window (to show a modal dialog)
         """
-        log.debug("configure() not overridden by derived class %s" % self._name_)
+        pass
 
     def is_configured(self, isSource, isTwoWay):
         """
@@ -224,7 +190,6 @@ class DataProviderBase(gobject.GObject):
         @returns: Dictionary of strings containing application settings
         @rtype: C{dict(string)}
         """
-        log.debug("get_configuration() not overridden by derived class %s" % self._name_)
         return {}
 
     def get_configuration_xml(self):
@@ -257,12 +222,10 @@ class DataProviderBase(gobject.GObject):
         Restores applications settings
         @param config: dictionary of dataprovider settings to restore
         """
-        log.debug("set_configuration() not overridden by derived class %s" % self._name_)
         for c in config:
             #Perform these checks to stop malformed xml from stomping on
             #unintended variables or posing a security risk by overwriting methods
             if getattr(self, c, None) != None and callable(getattr(self, c, None)) == False:
-                log.debug("Setting %s to %s" % (c, config[c]))
                 setattr(self,c,config[c])
             else:
                 log.warn("Not restoring %s setting: Exists=%s Callable=%s" % (
@@ -295,7 +258,6 @@ class DataProviderBase(gobject.GObject):
                             #fallback to string type
                             log.warn("Cannot convert string (%s) to native type %s\n" % (raw, vtype, traceback.format_exc()))
                             data = str(raw)
-                        log.debug("Read Setting: Name=%s Value=%s Type=%s" % (s.localName, data, type(data)))
                         settings[s.localName] = data
 
             try:
@@ -358,8 +320,13 @@ class DataProviderBase(gobject.GObject):
             return self._out_type_
         else:
             return "%s?%s" % (self._out_type_, Utils.encode_conversion_args(args))
-
-
+            
+    def get_name(self):
+        """
+        @returns: The DataProvider name, to be displayed in the UI
+        """
+        return self._name_
+        
 class DataSource(DataProviderBase):
     """
     Base Class for DataSources.
@@ -472,40 +439,32 @@ class DataProviderFactory(gobject.GObject):
     dynamic dataproviders become available at runtime.
     """
     __gsignals__ = {
-        "dataprovider-available" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [
+        "dataprovider-added" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [
             gobject.TYPE_PYOBJECT,      #Wrapper
             gobject.TYPE_PYOBJECT]),    #Class
-        "dataprovider-unavailable" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [
-            gobject.TYPE_STRING])     #Unique key
+        "dataprovider-removed" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [
+            gobject.TYPE_STRING])       #Unique key
     }
 
+    _module_type_ = "dataprovider-factory"
+
     def __init__(self, **kwargs):
         gobject.GObject.__init__(self)
 
-    def emit_added(self, klass, initargs=(), category=None):
-        if category == None:
-            category = getattr(klass, "_category_", conduit.dataproviders.CATEGORY_TEST)
+    def emit_added(self, klass, initargs, category, customKey=None):
         dpw = ModuleWrapper.ModuleWrapper(   
-                    name=getattr(klass, "_name_", ""),
-                    description=getattr(klass, "_description_", ""),
-                    icon_name=getattr(klass, "_icon_", ""),
-                    module_type=getattr(klass, "_module_type_", ""),
-                    category=category,
-                    in_type=getattr(klass, "_in_type_", ""),
-                    out_type=getattr(klass, "_out_type_", ""),
-                    filename=__file__,
-                    classname=klass.__name__,
+                    klass=klass,
                     initargs=initargs,
-                    module=None,
-                    enabled=True
+                    category=category
                     )
-        log.debug("DataProviderFactory %s: Emitting dataprovider-available for %s" % (self, dpw.get_key()))
-        self.emit("dataprovider-available", dpw, klass)
-        return dpw.get_key()
+        dpw.set_dnd_key(customKey)
+        log.debug("DataProviderFactory %s: Emitting dataprovider-added for %s" % (self, dpw.get_dnd_key()))
+        self.emit("dataprovider-added", dpw, klass)
+        return dpw.get_dnd_key()
 
     def emit_removed(self, key):
-        log.debug("DataProviderFactory %s: Emitting dataprovider-unavailable for %s" % (self, key))
-        self.emit("dataprovider-unavailable", key)
+        log.debug("DataProviderFactory %s: Emitting dataprovider-removed for %s" % (self, key))
+        self.emit("dataprovider-removed", key)
 
     def probe(self):
         pass
